﻿package game{
	import flash.display.Sprite;
	import flash.utils.Timer;
	import flash.utils.Dictionary;
	import flash.events.*;
	import flash.ui.Keyboard;
	public class MainGame extends Sprite {
		var myShip:MyShip;
		var mainTimer:Timer;
		private var _enemyList:Dictionary;
		private var _bulletList:Dictionary;
		private var _effectList:Dictionary;
		private var _stageData:StageData;
		private var _roundCount:int;
		private var _nextRoundCount:int;
		
		public function MainGame() {
			startGame();
		}
		
		public function startGame() {
			myShip=new MyShip(150,300,20);
			addChild(myShip.body);
			mainTimer=new Timer(40,0);
			mainTimer.addEventListener(TimerEvent.TIMER,onMainTimer);
			_enemyList=new Dictionary  ;
			_bulletList=new Dictionary  ;
			_effectList=new Dictionary  ;
			addEventListener(Event.ENTER_FRAME,drawGame);
			stage.addEventListener(KeyboardEvent.KEY_DOWN,onKey);
			stage.addEventListener(KeyboardEvent.KEY_UP,onKey);
			_stageData=new StageData  ;
			_stageData.addEventListener("parse_complete",onStageLoad);
			loadStage("stage1Data.txt");
		}
		
		function loadStage(fileName:String):void {
			_stageData.load(fileName);
		}
		
		private function onStageLoad(event:Event):void {
			_roundCount=0;
			executeCommand();
			mainTimer.start();
		}

		private function onMainTimer(event:TimerEvent):void {
			myShip.action();
			for each (var enemy:Unit in _enemyList) {
				enemy.action();
			}
			for each (var bullet:Unit in _bulletList) {
				bullet.action();
			}
			for each (var effect:Unit in _effectList) {
				effect.action();
			}
			checkCollision();
			if (_roundCount++ > _nextRoundCount) {
				executeCommand();
				_roundCount=0;
			}
		}
		
		private function checkCollision():void {
			for each (var enemy:Unit in _enemyList) {
				if (isCollision(enemy,myShip,20)) {
					addEffect(enemy);
					enemy.remove();
					myShip.setDamage(10);
				}
				for each (var bullet:Unit in _bulletList) {
					if (isCollision(enemy,bullet,40)) {
						addEffect(enemy);
						enemy.remove();
						bullet.remove();
						if (enemy == null || enemy.body.stage == null) {
							break;
						}
					}
				}
			}
		}
		
		private function isCollision(baseUnit:Unit,checkUnit:Unit,size:int):Boolean {
			var dx:int=baseUnit.x - checkUnit.x;
			var dy:int=baseUnit.y - checkUnit.y;
			var dist:Number=Math.sqrt(dx * dx + dy * dy);
			var result:Boolean=dist <= size;
			return result;
		}
		
		private function executeCommand():void {
			var command:Array=_stageData.getNext();
			switch (command[0]) {
				case "wait" :
					_nextRoundCount=int(command[1]);
					break;
				case "end" :
					clearStage();
					break;
				default :
					var useClass:Class=getUseClass(command[0]);
					addEnemy(useClass,int(command[1]),int(command[2]));
					executeCommand();
			}
		}
		
		private function getUseClass(className:String):Class {
			switch (className) {
				case "EnemyA" :
					return EnemyA;
				case "EnemyB" :
					return EnemyB;
				case "EnemyC" :
					return EnemyC;
				default :
					return EnemyA;
			}
		}

		private function drawGame(event:Event):void {
			myShip.draw();
			for each (var enemy:Unit in _enemyList) {
				enemy.draw();
			}
			for each (var bullet:Unit in _bulletList) {
				bullet.draw();
			}
			for each (var effect:Unit in _effectList) {
				effect.draw();
			}
		}
		
		private function onKey(event:KeyboardEvent) {
			switch (event.keyCode) {
				case Keyboard.RIGHT :
				case Keyboard.LEFT :
				case Keyboard.SPACE :
					var flag:Boolean=event.type == KeyboardEvent.KEY_DOWN;
					myShip.setKeyFlag(event.keyCode,flag);
					break;
			}
		}
		
		/*敵の追加/消去処理*/
		function addEnemy(unitClass:Class,newX:int,newY:int) {
			var enemy:Unit;
			if (unitClass == EnemyC) {
				enemy=new unitClass(myShip,newX,newY);
			} else {
				enemy=new unitClass(newX,newY);
			}
			addChild(enemy.body);
			enemy.draw();
			_enemyList[enemy]=enemy;
		}
		
		function removeEnemy(enemy:Unit) {
			removeChild(enemy.body);
			delete _enemyList[enemy];
		}
		
		/*弾の追加消去処理*/
		function addBullet() {
			var bullet:Bullet=new Bullet(myShip);
			addChild(bullet.body);
			bullet.draw();
			_bulletList[bullet]=bullet;
		}
		
		function removeBullet(bullet:Unit) {
			removeChild(bullet.body);
			delete _bulletList[bullet];
		}
		
		/*エフェクトの追加消去処理*/
		function addEffect(unit:Unit) {
			var effect=new BombEffect(unit);
			addChild(effect.body);
			effect.draw();
			_effectList[effect]=effect;
		}
		
		function removeEffect(effect:Unit) {
			removeChild(effect.body);
			delete _effectList[effect];
		}
		
		function clearStage():void {
			trace("ステージが終了しました");
			mainTimer.stop();
		}
		
		function gameOver():void {
			trace("ゲームオーバーです");
			mainTimer.stop();
		}
	}
}